/*
 * Copyright (C) 2004, 2005 Michael Dominic K. (http://www.mdk.org.pl)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Template is a collection of settings for a given export. xTML files
 * are created from templates. Templates are system wide + user defined. 
 * Template is a directory on the disk with some special files. 
 */

using System;
using System.IO;
using System.Collections;
using Util;

public class TemplateException : ApplicationException {

    public TemplateException(string _Msg) : base (_Msg)
    {
    }
    
}


public class Template {

    public string BaseFilename;
    public string Name; //Mandatory
    public string Description; //Mandatory
    public string Author = null;
    public string Charset = null; // UTF-8 by default
    public bool PreviewImage = false;  
    public bool StylesheetPresent = false;
    
    /////////////////////////////////////////////////
    
    /* This constructor creates a template from a given directory
     * path 
     * */
    public Template(string _PathName)
    {
        Debug.Message("HTMLExport.dll", "Template", String.Format("Trying to construct template from {0}", _PathName));
        
        // Check if this is a valid template
        if (!File.Exists(_PathName + ".html")) throw new TemplateException(".html file for this Template not found!");

        BaseFilename = _PathName;

        // Check if file is fine
        RCConfigFile __RCFile = new RCConfigFile(_PathName + ".rc");
        if (__RCFile.IsVariableDeclared("NAME") == false || __RCFile.IsVariableDeclared("DESCRIPTION") == false)
            throw new TemplateException("One of the mandatory variables missing in rc file!");

        // Get basic params
        Name = __RCFile.GetValue("NAME");
        Description = __RCFile.GetValue("DESCRIPTION");
        Author = __RCFile.GetValue("AUTHOR");
        Charset = __RCFile.GetValue("CHARSET");
        if (Charset == null) Charset = "UTF-8";

        // Extra
        if (File.Exists(_PathName + ".png")) PreviewImage = true;
        if (File.Exists(_PathName + ".css")) StylesheetPresent = true;
        
    }
    

    /* Scan a given dir for other dirs that main contain templates */
    public static ArrayList ScanDirForTemplates(string _Path)
    {
        Debug.Message("HTMLExport.dll", "Template", String.Format("Scanning {0} for templates", _Path));

        DirectoryInfo __DirInfo = new DirectoryInfo(_Path);
        // Nothing to do
        if (__DirInfo.Exists == false) return null;
        
        ArrayList __TemplatesList = new ArrayList();
        
        foreach (FileInfo __FileInfo in __DirInfo.GetFiles())
        {
            if (__FileInfo.Extension == ".rc") {
                try {
                    Template __T = new Template(_Path + "/" + Path.GetFileNameWithoutExtension(__FileInfo.Name));
                    __TemplatesList.Add(__T);
                } catch (TemplateException __TExcp) {
                    Debug.Message("HTMLExport.dll", "Template", String.Format("Error while loading template: {0}", __TExcp.Message));
                } catch (Exception __Excp) {
                    Debug.Exception("HTMLExport.dll", "Template", __Excp);
                }
            }   
                    
        }
        
        
        
        return __TemplatesList;
    }


    /* Generate snippets for this template */
    public void SplitToSnippets(out HTMLSnippet _Header, out HTMLSnippet _Footer, out HTMLSnippet _Row, out HTMLSnippet _OddRow) 
    {
        Debug.Message("HTMLExport.dll", "Template", String.Format("Spliting {0} to snippets", BaseFilename + ".html"));
        
        // First let's read the file
        // FIXME: We need to read as byte and convert encodings!
        StreamReader __Reader = File.OpenText(BaseFilename + ".html");
        
        string __CurrentLine = null;
        string __HTML = String.Empty;

        while ((__CurrentLine = __Reader.ReadLine()) != null) {
            __HTML += __CurrentLine + "\n";
        }
         
        // Now let's find the row part
        int __RowStartIndex = __HTML.IndexOf("<movie_row>");
        if (__RowStartIndex == -1) throw new TemplateException("<movie_row> tag not found in the template! Broken template!");
        int __RowEndIndex = __HTML.IndexOf("</movie_row>");
        if (__RowEndIndex == -1) throw new TemplateException("</movie_row> tag not found in the template! Broken template!");
       
        // Let's see if we have the odd row...
        int __OddRowStartIndex = __HTML.IndexOf("<movie_oddrow>");
        int __OddRowEndIndex = -1;
        
        if (__OddRowStartIndex != -1) {
            __OddRowEndIndex = __HTML.IndexOf("</movie_oddrow>");
            if (__OddRowEndIndex == -1) throw new TemplateException("</movie_oddrow> tag not found in the template! Broken template!");
        }

        // FIXME: Check for overlapping indexes...
        // FIXME: Check the order of tags

        // Let's calculate where the header ends...
        int __HeaderEndIndex = 0;
        if (__OddRowStartIndex != -1) __HeaderEndIndex = System.Math.Min(__OddRowStartIndex, __RowStartIndex);
            else __HeaderEndIndex = __RowStartIndex;

        // Let's calculate where the footer starts
        int __FooterStartIndex = 0;
        if (__OddRowEndIndex != -1) __FooterStartIndex = System.Math.Max(__OddRowEndIndex, __RowEndIndex);
            else __FooterStartIndex = __RowEndIndex;

        // Let's actually copy
        Debug.Message("HTMLExport.dll", "Template", "Creating Header snippet");
        _Header = new HTMLSnippet(__HTML.Substring(0, __HeaderEndIndex));

        Debug.Message("HTMLExport.dll", "Template", "Creating Row snippet");
        _Row = new HTMLSnippet(__HTML.Substring(__RowStartIndex, __RowEndIndex - __RowStartIndex).Replace("<movie_row>",null));
                
        if (__OddRowStartIndex != -1) {
            Debug.Message("HTMLExport.dll", "Template", "Creating Odd Row snippet");
            _OddRow = new HTMLSnippet(__HTML.Substring(__OddRowStartIndex, __OddRowEndIndex - __OddRowStartIndex).Replace("<movie_oddrow>", null));
        } else _OddRow = null;
        
        Debug.Message("HTMLExport.dll", "Template", "Creating Footer snippet");
        _Footer = new HTMLSnippet(__HTML.Substring(__FooterStartIndex, __HTML.Length - __FooterStartIndex).Replace("</movie_row>", null).Replace("</movie_oddrow>", null));
        
        
        // Cleanup
        __Reader.Close();
    }


    
} 
